<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

if ( ! class_exists( 'WC_Szamlazz_Health_Check', false ) ) :

	class WC_Szamlazz_Health_Check {

		//Init notices
		public static function init() {

      add_filter( 'debug_information', array( __CLASS__, 'debug_info' ) );
    	add_filter( 'site_status_tests', array(__CLASS__, 'status_tests') );
    	add_action( 'wp_ajax_health-check-wc-szamlazz_teszt', array(__CLASS__, 'status_tests_ajax') );
		}

		public static function debug_info($debug) {
      $szamlazz = array(
  			'wc_szamlazz' => array(
  				'label'       => __( 'Számlázz.hu', 'wc_szamlazz' ),
  				'description' => sprintf(
  					__(
  						'Diagnosztikai információk a Számlázz.hu WooCommerce integrációhoz kapcsolódóan. Ha kérdésed van vagy valami nem működik, mellékeld ezeket az adatokat is: <a href="%1$s" target="_blank" rel="noopener noreferrer">Támogatás</a>',
  						'wc_szamlazz'
  					),
  					esc_html( 'https://szamlazz.visztpeter.me/' )
  				),
  				'fields'      => self::debug_info_data(),
  			),
  		);
  		$debug = array_merge($debug, $szamlazz);
  		return $debug;
		}

    public static function status_tests($core_tests) {

      $core_tests['direct']['wc_szamlazz_curl'] = array(
        'label' => __( 'Számlázz.hu követelmények', 'wc_szamlazz' ),
        'test'  => function() {
          $settings = get_option( 'woocommerce_wc_szamlazz_settings', null );

          $result = array(
      			'label'       => 'Számlázz.hu követelmények',
      			'status'      => 'good',
      			'badge'       => array(
      				'label' => __( 'Számlázz.hu' ),
      				'color' => 'blue',
      			),
      			'description' => 'A weboldal és a tárhely minden követelményenek megfelel egy sikeres számlakészítéshez a Számlázz.hu WooCommerce bővítménnyel.',
      			'test'        => 'wc_szamlazz_php_version',
      		);

          //Check for cURL
          if(!function_exists('curl_version')) {
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = __('A <strong>WooCommerce + Szamlazz.hu</strong> bővítmény használatához a cURL funkció szükséges és úgy néz ki, ezen a tárhelyen nincs bekapcsolva. Ha nem tudod mi ez, kérd meg a tárhelyszolgáltatót, hogy kapcsolják be.', 'wc_szamlazz');
          }

          //Username/password
          if($settings['wc_szamlazz_api_key'] || defined( 'WC_SZAMLAZZ_AGENT_KULCS' )) {
            //all good
          } else {
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = __('A <strong>WooCommerce + Szamlazz.hu</strong> bővítmény használatához add meg a bővítmény beállításaiban a Számla Agent kulcsot.', 'wc_szamlazz');
            $result['actions'] = sprintf(
      				'<p><a href="%s" target="_blank" rel="noopener noreferrer">%s <span aria-hidden="true" class="dashicons dashicons-admin-generic"></span></a></p>',
      				esc_url( admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_szamlazz' ) ),
      				__( 'Bellítások', 'wc_szamlazz' )
      			);
          }

          return $result;
        }
      );

      //Debug mode is turned on
      $core_tests['direct']['wc_szamlazz_debug'] = array(
        'label' => __( 'Számlázz.hu debug mód', 'wc_szamlazz' ),
        'test'  => function() {
          $settings = get_option( 'woocommerce_wc_szamlazz_settings', null );

          $result = array(
      			'label'       => __('Számlázz.hu fejlesztői mód ki van kapcsolva', 'wc-szamlazz'),
      			'status'      => 'good',
      			'badge'       => array(
      				'label' => __( 'Számlázz.hu' ),
      				'color' => 'blue',
      			),
      			'description' => 'A Számlázz.hu WooCommerce fejlesztői módja ki van kapcsolva.',
      			'test'        => 'wc_szamlazz_check_debug_mode',
      		);

          //If debug mode is turned on
          if($settings['wc_szamlazz_debug'] && $settings['wc_szamlazz_debug'] == 'yes') {
            $result['label'] = __('Számlázz.hu fejlesztői mód be van kapcsolva', 'wc-szamlazz');
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = __('A <strong>WooCommerce + Szamlazz.hu</strong> bővítmény fejlesztői módja be van kapcsolva. Éles használat esetén ezt mindenképp kapcsold ki, amit megtehetsz a bővítmény beállításaiban.', 'wc_szamlazz');
            $result['actions'] = sprintf(
      				'<p><a href="%s" target="_blank" rel="noopener noreferrer">%s <span aria-hidden="true" class="dashicons dashicons-admin-generic"></span></a></p>',
      				esc_url( admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_szamlazz' ) ),
      				__( 'Bellítások', 'wc_szamlazz' )
      			);
          }

          return $result;
        }
      );

      $core_tests['async']['wc_szamlazz_generate'] = array(
        'label' => __( 'Számlázz.hu számlakészítés', 'wc_szamlazz' ),
        'test'  => 'wc_szamlazz_teszt',
      );

      return $core_tests;
  	}

    public static function status_tests_ajax() {
      global $wc_szamlazz;
      check_ajax_referer( 'health-check-site-status' );

      //Build Xml
      $szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlszamlakifiz xmlns="http://www.szamlazz.hu/xmlszamlakifiz" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlakifiz http://www.szamlazz.hu/docs/xsds/agentkifiz/xmlszamlakifiz.xsd"></xmlszamlakifiz>');
  		$szamla->appendXML($wc_szamlazz->get_authentication_xml_object());
  		$szamla->beallitasok->addChild('szamlaszam', '1');
  		$szamla->beallitasok->addChild('additiv', 'false');
      $kifizetes = $szamla->addChild('kifizetes');
      $kifizetes->addChild('datum', date('Y-m-d') );
  		$kifizetes->addChild('jogcim', '' );
  		$kifizetes->addChild('osszeg', 1);
  		$xml = $szamla->asXML();

  		//Get response from Számlázz.hu
  		$xml_response = $wc_szamlazz->xml_generator->generate($xml, rand(), 'action-szamla_agent_kifiz');

      $result = array(
        'label'       => __( 'Számlázz.hu összekapcsolás sikeres.', 'wc_szamlazz' ),
        'status'      => 'good',
        'badge'       => array(
          'label' => __( 'Számlázz.hu' ),
          'color' => 'blue',
        ),
        'description' => sprintf(
          '<p>%s</p>',
          __( "A Számlázz.hu WooCommerce bővítménynek sikerült kapcsolatot létesítenie a megadott felhasználónévvel és jelszóval. Úgy néz ki, minden rendben.", 'wc_szamlazz' )
        ),
        'actions'     => '',
        'test'        => 'wc_szamlazz_test_generate_invoice',
      );

      if($xml_response['error']) {
        if($xml_response['http_error']) {
          $result['label'] = __('Számlázz.hu összekapcsolás sikertelen', 'wc-szamlazz');
          $result['status'] = 'critical';
          $result['badge']['color'] = 'red';
          $result['description'] = sprintf(
            '<p>%s</p><p>%s</p>',
            __( "A Számlázz.hu WooCommerce bővítménynek nem sikerült kommunikálnia a Számlázz.hu szerverével. Lehet gond azzal, hogy a cURL kérelem nem működik a tárhelyeden/szervereden, vagy tiltva van a kommunikáció tűzfallal, esetleg a Számlázz.hu éppen nem működik. A pontos hibaüzenet ez volt:", 'wc_szamlazz' ),
            esc_html($xml_response['http_error'])
          );
        } else {
          if($xml_response['agent_error_code'] == 3) {
            $result['label'] = __('Számlázz.hu összekapcsolás sikertelen', 'wc-szamlazz');
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = sprintf(
              '<p>%s</p><p>%s</p>',
              __( "A Számlázz.hu WooCommerce bővítménynek nem sikerült kommunikálnia a Számlázz.hu szerverével, mert nem sikerült belépni. Valószínűleg rossz a megadott felhasználónév vagy a jelszó. A pontos hibaüzenet ez volt:", 'wc_szamlazz' ),
              esc_html($xml_response['agent_error'])
            );
          } else if($xml_response['agent_error_code'] == 136) {
            $result['label'] = __('Számlázz.hu összekapcsolás sikertelen', 'wc-szamlazz');
            $result['status'] = 'critical';
            $result['badge']['color'] = 'red';
            $result['description'] = sprintf(
              '<p>%s</p><p>%s</p>',
              __( "A Számlázz.hu WooCommerce bővítménynek nem sikerült kommunikálnia a Számlázz.hu szerverével, mert valamilyen okból nem használhatod az Agentet. Ezek lehetnek az okai: lejárt a díjcsomag, e-számla díj kiegyenlítése szükséges, elvárt utalással késel. Ezen esetekben lépj be a Számlázz.hu rendszerbe és a díjcsomagom menüben ellenőrizd a tartozásaidat. A pontos hibaüzenet ez volt:", 'wc_szamlazz' ),
              esc_html($xml_response['agent_error'])
            );
          }
        }
  		}

      wp_send_json_success($result);
  	}

    public static function debug_info_data() {
      $debug_info = array();

			//PRO verzió
			$debug_info['wc_szamlazz_pro_version'] = array(
				'label'   => __('PRO verzió', 'wc_szamlazz'),
				'value'   => get_option('_wc_szamlazz_pro_enabled')
			);

			//Invoice path
			$UploadDir = wp_upload_dir();
			$UploadURL = $UploadDir['basedir'];
			$location  = realpath($UploadURL . "/wc_szamlazz/");
			$debug_info['wc_szamlazz_path'] = array(
				'label'   => __('Útvonal', 'wc_szamlazz'),
				'value'   => $location
			);

			//IPN URL
			$settings_api = new WC_Szamlazz_Settings();
			$debug_info['wc_szamlazz_ipn'] = array(
				'label'   => __('IPN URL', 'wc_szamlazz'),
				'value'   => $settings_api->get_ipn_url(),
				'private' => true
			);

			//Payment options
			$payment_options = get_option('wc_szamlazz_payment_method_options');
			$debug_info['wc_szamlazz_payment_options'] = array(
				'label'   => __('Fizetési módok', 'wc_szamlazz'),
				'value'   => print_r($payment_options, true)
			);

			//Display saved settings
      $settings = get_option( 'woocommerce_wc_szamlazz_settings', null );
      $options = $settings_api->form_fields;
			unset($options['wc_szamlazz_username']);
			unset($options['wc_szamlazz_password']);
			unset($options['wc_szamlazz_api_key']);

      foreach ($options as $option_id => $option) {
				if(!in_array($option['type'], array('pro', 'title', 'payment_methods', 'accounting_details', 'ipn'))) {
					$debug_info[$option_id] = array(
						'label'   => $option['title'],
						'value'   => $settings[$option_id]
					);
				}
      }

      return $debug_info;
    }

  }

	WC_Szamlazz_Health_Check::init();

endif;
