<?php
/*
Plugin Name: Integration for Szamlazz.hu & WooCommerce
Plugin URI: http://visztpeter.me
Description: Számlázz.hu összeköttetés WooCommercehez
Author: Viszt Péter
Version: 3.4.8.2
WC requires at least: 3.0.0
WC tested up to: 3.6.5
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly


//Generate stuff on plugin activation
function wc_szamlazz_activate() {
	update_option('_wc_szamlazz_fresh_install', true);

	$upload_dir =  wp_upload_dir();

	$files = array(
		array(
			'base' 		=> $upload_dir['basedir'] . '/wc_szamlazz',
			'file' 		=> 'index.html',
			'content' 	=> ''
		)
	);

	foreach ( $files as $file ) {
		if ( wp_mkdir_p( $file['base'] ) && ! file_exists( trailingslashit( $file['base'] ) . $file['file'] ) ) {
			if ( $file_handle = @fopen( trailingslashit( $file['base'] ) . $file['file'], 'w' ) ) {
				fwrite( $file_handle, $file['content'] );
				fclose( $file_handle );
			}
		}
	}
}
register_activation_hook( __FILE__, 'wc_szamlazz_activate' );

class WC_Szamlazz {

	public static $plugin_prefix;
	public static $plugin_url;
	public static $plugin_path;
	public static $plugin_basename;
	public static $version;
	public $xml_generator = null;
	protected static $background_generator = null;

  //Construct
	public function __construct() {

		//Default variables
		self::$plugin_prefix = 'wc_szamlazz_';
		self::$plugin_basename = plugin_basename(__FILE__);
		self::$plugin_url = plugin_dir_url(self::$plugin_basename);
		self::$plugin_path = trailingslashit(dirname(__FILE__));
		self::$version = '3.4.8.2';

		//XML generator helper
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-xml-generator.php' );
		$this->xml_generator = new WC_Szamlazz_Xml_Generator();

		//Plugin loaded
		add_action( 'plugins_loaded', array( $this, 'init' ) );

  }

	//Load plugin stuff
	public function init() {

		//Background invoice generator
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-background-generator.php' );
		self::$background_generator = new WC_Szamlazz_Background_Generator();

		// Load includes
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-settings.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-admin-notices.php' );
		require_once( plugin_dir_path( __FILE__ ) . 'includes/class-product-options.php' );

		//Health check for WP 5.2+
		global $wp_version;
		if ( version_compare( $wp_version, '5.2-alpha', 'ge' ) ) {
			require_once( plugin_dir_path( __FILE__ ) . 'includes/class-health-check.php' );
		}

		//Plugin links
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $this, 'plugin_action_links' ) );

		//Settings page
		add_filter( 'woocommerce_integrations', array( $this, 'add_integration' ) );

		add_action( 'admin_init', array( $this, 'wc_szamlazz_admin_init' ) );

		add_action( 'add_meta_boxes', array( $this, 'wc_szamlazz_add_metabox' ), 10, 2 );

		//Ajax functions
		add_action( 'wp_ajax_wc_szamlazz_generate_invoice', array( $this, 'generate_invoice_with_ajax' ) );
		add_action( 'wp_ajax_wc_szamlazz_generate_receipt', array( $this, 'generate_receipt_with_ajax' ) );
		add_action( 'wp_ajax_wc_szamlazz_complete', array( $this, 'generate_invoice_complete_with_ajax' ) );
		add_action( 'wp_ajax_wc_szamlazz_sztorno', array( $this, 'generate_invoice_sztorno_with_ajax' ) );
		add_action( 'wp_ajax_wc_szamlazz_sztorno_receipt', array( $this, 'generate_receipt_sztorno_with_ajax' ) );
		add_action( 'wp_ajax_wc_szamlazz_already', array( $this, 'wc_szamlazz_already' ) );
		add_action( 'wp_ajax_wc_szamlazz_already_back', array( $this, 'wc_szamlazz_already_back' ) );

		//Create a hook based on the status setup in settings to auto-generate invoice
		if(get_option('_wc_szamlazz_pro_enabled')) {
			$order_auto_status = str_replace( 'wc-', '', $this->get_option('wc_szamlazz_auto_status', 'completed') );
			add_action( 'woocommerce_order_status_'.$order_auto_status, array( $this, 'on_order_complete' ) );
			add_action( 'woocommerce_checkout_order_processed', array( $this, 'on_order_processing' ) );

			$order_auto_sztorno = str_replace( 'wc-', '', $this->get_option('wc_szamlazz_auto_sztorno', 'no') );
			if($order_auto_sztorno != 'no') {
				add_action( 'woocommerce_order_status_'.$order_auto_sztorno, array( $this, 'on_order_deleted' ) );
			}
		}

		//Order list button
		add_action( 'woocommerce_admin_order_actions_end', array( $this, 'add_listing_actions' ) );
		add_filter('woocommerce_my_account_my_orders_actions', array( $this, 'szamlazz_download_button' ), 10,2);

		if($this->get_option('wc_szamlazz_vat_number_form') == 'yes') {
			add_filter( 'woocommerce_checkout_fields' , array( $this, 'add_vat_number_checkout_field' ) );
			add_filter( 'woocommerce_before_checkout_form' , array( $this, 'add_vat_number_info_notice' ) );
			add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_vat_number' ) );
			add_action( 'woocommerce_admin_order_data_after_billing_address', array( $this, 'display_vat_number' ) );

			if($this->get_option('wc_szamlazz_vat_number_validate') == 'yes') {
				add_action( 'woocommerce_after_checkout_validation', array( $this, 'vat_number_validate' ), 10, 2);
			}
		}

		add_action( 'init', array( $this, 'wc_szamlazz_ipn_process' ) );

		//Frontend scripts & css
		if(($this->get_option('wc_szamlazz_nyugta') == 'yes' && get_option('_wc_szamlazz_pro_enabled')) || $this->get_option('wc_szamlazz_vat_number_validate') == 'yes') {
			add_action( 'wp_enqueue_scripts', array( $this, 'frontend_css' ));
		}

		//E-Nyugta
		if($this->get_option('wc_szamlazz_nyugta') == 'yes' && get_option('_wc_szamlazz_pro_enabled')) {
			add_filter( 'woocommerce_checkout_fields' , array( $this, 'add_nyugta_checkout_field' ) );
			add_action( 'wp_ajax_wc_szamlazz_nyugta_check', array( $this, 'wc_szamlazz_nyugta_check' ) );
			add_action( 'wp_ajax_nopriv_wc_szamlazz_nyugta_check', array( $this, 'wc_szamlazz_nyugta_check' ) );
			add_filter( 'woocommerce_checkout_get_value' , array( $this, 'nyugta_get_checkout_value' ), 10, 2 );
			add_action( 'woocommerce_cart_updated', array( $this, 'store_nyugta_session_data') );
			add_action( 'woocommerce_checkout_update_order_meta', array( $this, 'save_nyugta_session_data') );
		}

		//Attach invoice to emails
		if($this->get_option('wc_szamlazz_attachment') == 'yes') {
			add_filter( 'woocommerce_email_attachments', array( $this, 'email_attachment'), 10, 3 );
		}

	}

	//Integration page
	public function add_integration( $integrations ) {
		$integrations[] = 'WC_Szamlazz_Settings';
		return $integrations;
	}

  //Add CSS & JS
	public function wc_szamlazz_admin_init() {
		wp_enqueue_script( 'szamlazz_js', plugins_url( '/assets/js/admin.js',__FILE__ ), array('jquery'), WC_Szamlazz::$version, TRUE );
		wp_enqueue_style( 'szamlazz_css', plugins_url( '/assets/css/admin.css',__FILE__ ), array(), WC_Szamlazz::$version );

		$wc_szamlazz_local = array( 'loading' => plugins_url( '/assets/images/ajax-loader.gif',__FILE__ ) );
		wp_localize_script( 'szamlazz_js', 'wc_szamlazz_params', $wc_szamlazz_local );
  }

	//Frontend CSS & JS
	public function frontend_css() {
		wp_enqueue_style( 'wc_szamlazz_css', plugins_url( '/assets/css/frontend.css',__FILE__ ), array(), WC_Szamlazz::$version );
		wp_enqueue_script( 'wc_szamlazz_js', plugins_url( '/assets/js/frontend.js',__FILE__ ), array(), WC_Szamlazz::$version );
	}

	//Meta box on order page
	public function wc_szamlazz_add_metabox( $post_type, $post ) {
		add_meta_box('wc_szamlazz_metabox', 'Számlázz.hu számla', array( $this, 'render_meta_box_content' ), 'shop_order', 'side');
		$order = wc_get_order($post->ID);
		if($order) {
			$vat_number_data = $order->get_meta('_wc_szamlazz_adoszam_data');
			if($vat_number_data) {
				add_meta_box('wc_szamlazz_vat_number_metabox', 'Adószám információ', array( $this, 'render_meta_box_content_vat_number' ), 'shop_order', 'side');
			}
		}
	}

	//Render metabox content
	public function render_meta_box_content($post) {
		$order = wc_get_order($post->ID);
		include( dirname( __FILE__ ) . '/includes/views/html-metabox.php' );
	}

	//Vat number metabox content
	public function render_meta_box_content_vat_number($post) {
		$order = wc_get_order($post->ID);
		$vat_number_data = $order->get_meta('_wc_szamlazz_adoszam_data');
		include( dirname( __FILE__ ) . '/includes/views/html-metabox-vat.php' );
	}

	//Generate Invoice with Ajax
	public function generate_invoice_with_ajax() {
		check_ajax_referer( 'wc_generate_invoice', 'nonce' );
		if( true ) {
			$order_id = intval($_POST['order']);
			if($this->get_option('wc_szamlazz_szallitolevel') == 'yes' && $_POST['delivery_note'] == 'off' && !$this->is_delivery_note_generated($order_id)) {
				$return_info_szallito = $this->generate_invoice($order_id, false, true);
				$return_info = $this->generate_invoice($order_id);
				$return_info['link_delivery'] = $return_info_szallito['link'];
				wp_send_json_success($return_info);
			} else {
				$return_info = $this->generate_invoice($order_id);

				//If credit entry enabled for payment method
				$order = new WC_Order($order_id);
				$payment_method = $this->get_order_property('payment_method',$order);

				if($this->get_option('wc_szamlazz_payment_method_options')) {
					if($this->check_payment_method_options($payment_method, 'complete')) {
						$this->generate_invoice_complete($order_id);
					}
				} else {
					if($this->get_option('wc_szamlazz_auto_completes') && in_array($payment_method,$this->get_option('wc_szamlazz_auto_completes'))) {
						$this->generate_invoice_complete($order_id);
					}
				}

				wp_send_json_success($return_info);
			}
		}
	}

	//Generate XML for Szamla Agent
	public function generate_invoice($orderId,$payment_request = false,$delivery_note = false) {
		global $wpdb, $woocommerce;
		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlszamla xmlns="http://www.szamlazz.hu/xmlszamla" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlszamla xmlszamla.xsd"></xmlszamla>');

		//If custom details
		if(isset($_POST['note']) && isset($_POST['deadline']) && isset($_POST['completed'])) {
			$note = sanitize_text_field($_POST['note']);
			$deadline = intval($_POST['deadline']);
			$complated_date = sanitize_text_field($_POST['completed']);
		} else {
			$note = $this->get_option('wc_szamlazz_note');
			$deadline = $this->get_payment_method_deadline($this->get_order_property('payment_method',$order));
			$complated_date = date('Y-m-d');
		}

		//Replace customer email in note
		$note_replacements = array('{customer_email}' => $this->get_order_property('billing_email',$order), '{customer_phone}' => $this->get_order_property('billing_phone',$order));
		$note = str_replace( array_keys( $note_replacements ), array_values( $note_replacements ), $note);

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());
		$electronic_invoice_type = $this->get_invoice_type($order);
		$szamla->beallitasok->addChild('eszamla', $electronic_invoice_type);
		$szamla->beallitasok->addChild('szamlaLetoltes', 'true');

		//Invoice details
		$fejlec = $szamla->addChild('fejlec');
		$fejlec->addChild('keltDatum', date('Y-m-d') );
		$fejlec->addChild('teljesitesDatum', $complated_date );
		$fejlec->addChild('fizetesiHataridoDatum', ($deadline) ? date('Y-m-d', strtotime('+'.$deadline.' days')) : date('Y-m-d'));
		$fizmod = $this->get_order_property('payment_method_title',$order);
		if(!$fizmod || $fizmod == '') {
			$fizmod = '-';
		}
		$fejlec->addChild('fizmod', $fizmod);
		$order_currency = method_exists( $order, 'get_currency' ) ? $order->get_currency() : $order->get_order_currency();
		$fejlec->addChild('penznem', $order_currency);
		$fejlec->addChild('szamlaNyelve', 'hu');
		$fejlec->addChild('megjegyzes', $note);
		if($order_currency != 'HUF') {
			//if the base currency is not HUF, we should define currency rates
			$fejlec->addChild('arfolyamBank', 'MNB');
		}
		$fejlec->addChild('rendelesSzam', $order->get_order_number());
		$fejlec->addChild('elolegszamla', 'false');
		$fejlec->addChild('vegszamla', 'false');

		//Díjbekérő
		if($payment_request) {
			$fejlec->addChild('dijbekero', 'true');
		} else {
			if(isset($_POST['request']) && $_POST['request'] == 'on') {
				$fejlec->addChild('dijbekero', 'true');
				$payment_request = true;
			} else {
				$fejlec->addChild('dijbekero', 'false');
			}
		}

		//Ha szállítólevél
		$delivery_note_auto = false;
		if($delivery_note) {
			$fejlec->addChild('szallitolevel', 'true');
			$delivery_note_auto = true;
		} else if(isset($_POST['delivery_note']) && $_POST['delivery_note'] == 'on') {
			$fejlec->addChild('szallitolevel', 'true');
			$delivery_note = true;
		}

		//If előleg számla, csak díjbekérő esetén
		if($payment_request && $this->get_option('wc_szamlazz_eloleg') == 'yes') {
			$fejlec->elolegszamla = 'true';
			$fejlec->dijbekero = 'false';
		}

		//Előtag
		if($this->get_option('wc_szamlazz_elotag')) {
			$fejlec->szamlaszamElotag = $this->get_option('wc_szamlazz_elotag');
		}

		//Számla nyelv
		if($this->get_option('wc_szamlazz_nyelv')) {
			$fejlec->szamlaNyelve = $this->get_option('wc_szamlazz_nyelv');
		}

		//Language based on WPML
		if($this->get_option('wc_szamlazz_nyelv_wpml') == 'yes') {
			$wpml_lang_code = get_post_meta( $orderId, 'wpml_language', true );
			if(!$wpml_lang_code && function_exists('pll_get_post_language')){
				$wpml_lang_code = pll_get_post_language($orderId, 'locale');
			}
			if($wpml_lang_code && in_array($wpml_lang_code, array('hu', 'de', 'en', 'it', 'fr', 'hr', 'ro', 'sk', 'es', 'pl', 'cz'))) $fejlec->szamlaNyelve = $wpml_lang_code;
		}

		//Seller details
		$elado = $szamla->addChild('elado');
		$elado->addChild('bank','');
		$elado->addChild('bankszamlaszam','');

		//Customer details
		$vevo = $szamla->addChild('vevo');
		$vevo->addChild('nev', ($this->get_order_property('billing_company',$order) ? htmlspecialchars($this->get_order_property('billing_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('billing_first_name',$order).' '.$this->get_order_property('billing_last_name',$order)) );

		$vevo->addChild('orszag', WC()->countries->countries[ $this->get_order_property('billing_country',$order) ]);
		$vevo->addChild('irsz',$this->get_order_property('billing_postcode',$order));
		$vevo->addChild('telepules',$this->get_order_property('billing_city',$order));
		$vevo->addChild('cim',$this->get_order_property('billing_address_1',$order));
		$vevo->addChild('email',$this->get_order_property('billing_email',$order));

		//Switch name if needed
		if($this->get_option('wc_szamlazz_nev_csere') == 'yes') {
			$vevo->nev = ($this->get_order_property('billing_company',$order) ? htmlspecialchars($this->get_order_property('billing_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('billing_last_name',$order).' '.$this->get_order_property('billing_first_name',$order));
		}

		//Add name after company name
		if($this->get_order_property('billing_company',$order) && $this->get_option('wc_szamlazz_cegnev') == 'yes') {
			$vevo->nev .= ' - '.$this->get_order_property('billing_last_name',$order).' '.$this->get_order_property('billing_first_name',$order);
		}

		//Set if we don't need to send emails to customers
		if($this->get_option('wc_szamlazz_auto_email', 'yes') == 'yes') {
			$vevo->addChild('sendEmail', 'true');
		} else {
			$vevo->addChild('sendEmail', 'false');
		}

		//VAT number
		$adoszam = get_post_meta( $orderId, 'adoszam', true );
		if(!$adoszam) $adoszam = '';
		$vevo->addChild('adoszam', $adoszam);
		$vevo->addChild('adoszamEU', '');

		//Customer Shipping details if needed
		if($order->get_shipping_methods() && $this->get_option('wc_szamlazz_hide_shipping_info', 'no') != 'yes') {
			$vevo->addChild('postazasiNev', ($this->get_order_property('shipping_company',$order) ? htmlspecialchars($this->get_order_property('shipping_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('shipping_first_name',$order).' '.$this->get_order_property('shipping_last_name',$order)) );
			$vevo->addChild('postazasiIrsz',$this->get_order_property('shipping_postcode',$order));
			$vevo->addChild('postazasiTelepules',$this->get_order_property('shipping_city',$order));
			$vevo->addChild('postazasiCim',$this->get_order_property('shipping_address_1',$order));
		}

		//Switch name if needed
		if($this->get_option('wc_szamlazz_nev_csere') == 'yes') {
			$vevo->postazasiNev = ($this->get_order_property('shipping_company',$order) ? htmlspecialchars($this->get_order_property('shipping_company',$order), ENT_XML1, 'UTF-8') : $this->get_order_property('shipping_last_name',$order).' '.$this->get_order_property('shipping_first_name',$order));
		}

		//Accounting details
		if($this->get_option('wc_szamlazz_accounting_details_enabled', 'no') == 'yes' && $this->get_option('wc_szamlazz_accounting_details_vevo_azonosito', 'no') == 'yes') {
			$vevo->addChild('vevoFokonyv');
			$vevo->vevoFokonyv->addChild('vevoAzonosito', $order->get_customer_id());
			$vevo->vevoFokonyv->addChild('vevoFokonyviSzam', $order->get_customer_id());
		}

		//Phone number
		$vevo->addChild('telefonszam',$this->get_order_property('billing_phone',$order));

		//Order Items
		$tetelek = $szamla->addChild('tetelek');

		foreach( $order_items as $termek ) {
			$product_id = $termek['product_id'];

			$sku = ($termek->get_product()) ? $termek->get_product()->get_sku() : '';
			$tetel = $tetelek->addChild('tetel');
			if($termek->get_product() && $termek->get_product()->get_meta('wc_szamlazz_tetel_nev') && $termek->get_product()->get_meta('wc_szamlazz_tetel_nev') != '') {
				$tetel->addChild('megnevezes', htmlspecialchars($termek->get_product()->get_meta('wc_szamlazz_tetel_nev')));
			} else {
				$tetel->addChild('megnevezes',htmlspecialchars($termek->get_name()));
			}
			$tetel->addChild('azonosito',$sku);
			$tetel->addChild('mennyiseg',$termek->get_quantity());

			if($termek->get_product() && $termek->get_product()->get_meta('wc_szamlazz_mennyisegi_egyseg')) {
				$tetel->addChild('mennyisegiEgyseg', $termek->get_product()->get_meta('wc_szamlazz_mennyisegi_egyseg'));
			} else if($this->get_option('wc_szamlazz_mennyisegi_egyseg')) {
				$tetel->addChild('mennyisegiEgyseg', $this->get_option('wc_szamlazz_mennyisegi_egyseg'));
			} else {
				$tetel->addChild('mennyisegiEgyseg', '');
			}

			if($this->get_option('wc_szamlazz_coupon_type') == 'yes') {
				if(round($termek->get_subtotal(),2) == 0) {
					$tetel->addChild('nettoEgysegar',0);
					$tetel->addChild('afakulcs',0);
				} else {
					$tetel->addChild('nettoEgysegar', round($termek->get_subtotal(),2)/$termek->get_quantity());
					$tetel->addChild('afakulcs', $this->get_order_item_tax_label($order, $termek) );
				}
			} else {
				if(round($termek->get_total(),2) == 0) {
					$tetel->addChild('nettoEgysegar',0);
					$tetel->addChild('afakulcs',0);
				} else {
					$tetel->addChild('nettoEgysegar', round($termek->get_total(),2)/$termek->get_quantity());
					$tetel->addChild('afakulcs', $this->get_order_item_tax_label($order, $termek) );
				}
			}

			if($this->get_option('wc_szamlazz_coupon_type') == 'yes') {
				$tetel->addChild('nettoErtek', round($termek->get_subtotal(),2));
				$tetel->addChild('afaErtek', round($termek->get_subtotal_tax(),2));
				$tetel->addChild('bruttoErtek', round($termek->get_subtotal(),2)+round($termek->get_subtotal_tax(),2));
			} else {
				$tetel->addChild('nettoErtek', round($termek->get_total(),2));
				$tetel->addChild('afaErtek', round($termek->get_total_tax(),2));
				$tetel->addChild('bruttoErtek', round($termek->get_total(),2)+round($termek->get_total_tax(),2));
			}

			//Item note
			$note = '';

			//Show variation details if needed
			$product_name = $termek->get_name();
			if(strpos($product_name, ' - ') === false) {
				$note = strip_tags(wc_display_item_meta( $termek, array(
					'before'    => "\n- ",
					'separator' => "\n- ",
					'after'     => "",
					'echo'      => false,
					'autop'     => false,
				)));
				if($note != '') $note .= "\n";
			}

			//Custom note
			if($termek->get_product() && $termek->get_product()->get_meta('wc_szamlazz_megjegyzes')) {
				$note .= $termek->get_product()->get_meta('wc_szamlazz_megjegyzes');
			}

			//If we need to show sale price in the note
			if($this->get_option('wc_szamlazz_discount_note') && $termek->get_product() && $termek->get_product()->is_on_sale()) {
				$sale_price = (float)$termek->get_product()->get_sale_price();
				$net_unit_price = $termek->get_total()/$termek->get_quantity();
				$gross_unit_price = ($termek->get_total()+$termek->get_total_tax())/$termek->get_quantity();

				if($sale_price == $net_unit_price || $sale_price == $gross_unit_price) {
					if(get_option( 'woocommerce_prices_include_tax') == 'no') {
						$afakulcs = 1+$termek->get_total_tax()/$termek->get_total();
					} else {
						$afakulcs = 1;
					}
					$regular_price = $termek->get_product()->get_regular_price()*$afakulcs;
					$original_price = $regular_price*$termek->get_quantity();
					$applied_sale = $original_price-($termek->get_total()+$termek->get_total_tax());
					$discounted_price = $termek->get_total()+$termek->get_total_tax();
					$discount_note = $this->get_option('wc_szamlazz_discount_note');
					$discount_note_replacements = array('{eredeti_ar}' => wc_price($original_price), '{kedvezmeny_merteke}' => wc_price($applied_sale), '{kedvezmenyes_ar}' => wc_price($discounted_price));
					$discount_note = str_replace( array_keys( $discount_note_replacements ), array_values( $discount_note_replacements ), $discount_note);
					$discount_note = strip_tags($discount_note);
					$discount_note = html_entity_decode($discount_note);
					$note .= $discount_note;
				}
			}

			$tetel->addChild('megjegyzes', $note);

			//See if we can get accounting details
			$accounting_details = $this->get_accounting_details($order, get_the_terms( $product_id, 'product_cat' ));
			if($accounting_details) {
				$tetel->addChild('tetelFokonyv');
				$tetel->tetelFokonyv->addChild('gazdasagiEsem', $accounting_details['gazd_esem']);
				$tetel->tetelFokonyv->addChild('gazdasagiEsemAfa', $accounting_details['afa_gazd_esem']);
				$tetel->tetelFokonyv->addChild('arbevetelFokonyviSzam', $accounting_details['fokonyvi_szam']);
				$tetel->tetelFokonyv->addChild('afaFokonyviSzam', $accounting_details['afa_fokonyvi_szam']);
			}

		}

		//Shipping
		foreach( $order->get_items( 'shipping' ) as $item_id => $shipping_item_obj ){
			$order_shipping = $shipping_item_obj->get_total();
			$order_shipping_tax = $shipping_item_obj->get_total_tax();
			if($this->get_option('wc_szamlazz_hide_free_shipping') == 'yes' && $order_shipping == 0) {
				continue;
			}

			$tetel = $tetelek->addChild('tetel');
			$tetel->addChild('megnevezes', htmlspecialchars($shipping_item_obj->get_method_title()));
			$tetel->addChild('mennyiseg','1');
			$tetel->addChild('mennyisegiEgyseg','');
			$tetel->addChild('nettoEgysegar',round($order_shipping,2));
			if($order_shipping == 0) {
				$tetel->addChild('afakulcs','0');
			} else {
				$tetel->addChild('afakulcs', $this->get_order_shipping_tax_label($order, $shipping_item_obj) );
			}
			$tetel->addChild('nettoErtek',round($order_shipping,2));
			$tetel->addChild('afaErtek',round($order_shipping_tax,2));
			$tetel->addChild('bruttoErtek',round($order_shipping,2)+round($order_shipping_tax,2));
			$tetel->addChild('megjegyzes','');

			//See if we can get accounting details
			$accounting_details = $this->get_accounting_details($order, $shipping_item_obj->get_method_id());
			if($accounting_details) {
				$tetel->addChild('tetelFokonyv');
				$tetel->tetelFokonyv->addChild('gazdasagiEsem', $accounting_details['gazd_esem']);
				$tetel->tetelFokonyv->addChild('gazdasagiEsemAfa', $accounting_details['afa_gazd_esem']);
				$tetel->tetelFokonyv->addChild('arbevetelFokonyviSzam', $accounting_details['fokonyvi_szam']);
				$tetel->tetelFokonyv->addChild('afaFokonyviSzam', $accounting_details['afa_fokonyvi_szam']);
			}
		}

		//Extra Fees
		$fees = $order->get_fees();
		if(!empty($fees)) {
			foreach( $fees as $fee ) {
				$tetel = $tetelek->addChild('tetel');
				$tetel->addChild('megnevezes',htmlspecialchars($fee["name"]));
				$tetel->addChild('mennyiseg',1);
				$tetel->addChild('mennyisegiEgyseg','');
				if(round($fee->get_total(),2) == 0) {
					$tetel->addChild('nettoEgysegar',0);
					$tetel->addChild('afakulcs',0);
				} else {
					$tetel->addChild('nettoEgysegar',round($fee["line_total"],2));
					$tetel->addChild('afakulcs', $this->get_order_shipping_tax_label($order, $fee) );
				}
				$tetel->addChild('nettoErtek',round($fee["line_total"],2));
				$tetel->addChild('afaErtek',round($fee["line_tax"],2));
				$tetel->addChild('bruttoErtek',round($fee["line_total"],2)+round($fee["line_tax"],2));
				$tetel->addChild('megjegyzes','');
			}
		}

		//Discount
		$order_discount = method_exists( $order, 'get_discount_total' ) ? $order->get_discount_total() : $order->order_discount;
		if ( $order_discount > 0 ) {
			$discout_details = $this->get_coupon_invoice_item_details($order);

			//If coupon is a separate item
			if($this->get_option('wc_szamlazz_coupon_type') == 'yes') {

				$tetel = $tetelek->addChild('tetel');
				$tetel->addChild('megnevezes', $discout_details["title"]);
				$tetel->addChild('mennyiseg','1');
				$tetel->addChild('mennyisegiEgyseg', $this->get_option('wc_szamlazz_mennyisegi_egyseg'));
				$tetel->addChild('nettoEgysegar', -round($order->get_total_discount(),2));
				$tetel->addChild('afakulcs', round( ($order->get_discount_tax()/$order->get_total_discount()) * 100 ) );
				$tetel->addChild('nettoErtek', -round($order->get_total_discount(),2));
				$tetel->addChild('afaErtek', -round($order->get_discount_tax(),2));
				$tetel->addChild('bruttoErtek', -(round($order->get_total_discount(),2)+round($order->get_discount_tax(),2)));
				$tetel->addChild('megjegyzes', $discout_details["desc"]);

			} else {
				//Add space if theres already something in the comment
				if($szamla->fejlec->megjegyzes) {
					$szamla->fejlec->megjegyzes .= ' ';
				}

				$szamla->fejlec->megjegyzes .= $discout_details["desc"];
			}
		}

		//Change afakulcs if needed
		if($this->get_option('wc_szamlazz_afakulcs')) {
			foreach($tetelek->tetel as $tetel) {
				$tetel->afakulcs = $this->get_option('wc_szamlazz_afakulcs');
			}
		}

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml',$szamla,$order);
		$xml = $xml_szamla->asXML();

		//Get response from Számlázz.hu
		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-xmlagentxmlfile');

		if($xml_response['error']) {

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu számlakészítés sikertelen! Agent hibakód: ', 'wc-szamlazz' ).urldecode($xml_response['agent_error']) );

			//Run action
			do_action('wc_szamlazz_after_invoice_error', $order, $xml_response);

			return $xml_response;

		} else {

			//Get the Invoice ID from the response header
			$szlahu_szamlaszam = $this->xml_generator->get_invoice_id($xml_response['header_array']);

			//Download & Store PDF - generate a random file name so it will be downloadable later only by you
			$pdf_file_name = $this->xml_generator->save_pdf_file($orderId);

			//Create response
			$xml_response['invoice_name'] = $szlahu_szamlaszam;

			//We sent an email?
			$auto_email_sent = $this->get_option('wc_szamlazz_auto_email', 'yes');

			if($payment_request) {

				//Text response
				$xml_response['messages'][] = ($auto_email_sent == 'yes') ? __('Díjbekérő sikeresen létrehozva és elküldve a vásárlónak emailben.','wc-szamlazz') : __('Díjbekérő sikeresen létrehozva.','wc-szamlazz');

				//Update order notes
				$order->add_order_note( __( 'Szamlazz.hu díjbekérő sikeresen létrehozva. A számla sorszáma: ', 'wc-szamlazz' ).$szlahu_szamlaszam );

				//Store as a custom field
				update_post_meta( $orderId, '_wc_szamlazz_dijbekero', $szlahu_szamlaszam );
				update_post_meta( $orderId, '_wc_szamlazz_dijbekero_pdf', $pdf_file_name );

			} else if($delivery_note) {

				//Text response
				$xml_response['messages'][] = __('Szállítólevél sikeresen létrehozva.','wc-szamlazz');

				//Update order notes
				$order->add_order_note( __( 'Szamlazz.hu szállítólevél sikeresen létrehozva. A szállítólevél sorszáma: ', 'wc-szamlazz' ).$szlahu_szamlaszam );

				//Store as a custom field
				update_post_meta( $orderId, '_wc_szamlazz_szallitolevel', $szlahu_szamlaszam );
				update_post_meta( $orderId, '_wc_szamlazz_szallitolevel_pdf', $pdf_file_name );

			} else {

				//Text response
				$xml_response['messages'][] = ($auto_email_sent == 'yes') ? __('Számla sikeresen létrehozva és elküldve a vásárlónak emailben.','wc-szamlazz') : __('Számla sikeresen létrehozva.','wc-szamlazz');

				//Update order notes
				$order->add_order_note( __( 'Szamlazz.hu számla sikeresen létrehozva. A számla sorszáma: ', 'wc-szamlazz' ).$szlahu_szamlaszam );

				//Store as a custom field
				update_post_meta( $orderId, '_wc_szamlazz', $szlahu_szamlaszam );
				update_post_meta( $orderId, '_wc_szamlazz_pdf', $pdf_file_name );

			}

			//Return the download url
			$button_label = __('Számla megtekintése','wc-szamlazz');
			if($payment_request) {
				$pdf_url = $this->generate_download_link($orderId,'dijbekero');
			} else if($delivery_note) {
				$pdf_url = $this->generate_download_link($orderId,'szallitolevel');
				$button_label = __('Szállítólevél megtekintése','wc-szamlazz');
			} else {
				$pdf_url = $this->generate_download_link($orderId);
			}

			$xml_response['link'] = '<p><a href="'.$pdf_url.'" id="wc_szamlazz_download" class="button button-primary" target="_blank">'.$button_label.'</a></p>';

			do_action('wc_szamlazz_after_invoice_success', $order, $xml_response, $szlahu_szamlaszam, $pdf_url);

			return $xml_response;

		}
	}

	//Autogenerate invoice
	public function on_order_complete( $order_id ) {

		//Only generate invoice, if it wasn't already generated & only if automatic invoice is enabled
		if($this->get_option('wc_szamlazz_auto') == 'yes') {
			if(!$this->is_invoice_generated($order_id)) {
				$return_info = false;

				//Check if we need to generate an invoice or a receipt
				if(get_post_meta( $order_id, '_wc_nyugta_type', true ) == 'receipt') {
					$return_info = $this->generate_receipt($order_id);
				} else {

					//Create szállítólevél automatically if needed
					if($this->get_option('wc_szamlazz_szallitolevel') == 'yes') {
						$return_info = $this->generate_invoice($order_id, false, true);
					}

					//Check if we generate this invoice deferred
					if($this->get_option('wc_szamlazz_defer') == 'yes') {
						self::$background_generator->push_to_queue(
							array(
								'invoice_type' => 'final',
								'order_id' => $order_id
							)
						);
					} else {
						$return_info = $this->generate_invoice($order_id);
					}

					//If credit entry enabled for payment method
					$order = new WC_Order($order_id);
					$payment_method = $this->get_order_property('payment_method',$order);

					if($this->get_option('wc_szamlazz_payment_method_options')) {
						if($this->check_payment_method_options($payment_method, 'complete')) {
							if($this->get_option('wc_szamlazz_defer') == 'yes') {
								self::$background_generator->push_to_queue(
									array(
										'invoice_type' => 'complete',
										'order_id' => $order_id
									)
								);
							} else {
								if($this->is_invoice_generated($order_id)) {
									$return_info = $this->generate_invoice_complete($order_id);
								}
							}
						}
					} else {
						if($this->get_option('wc_szamlazz_auto_completes') && in_array($payment_method,$this->get_option('wc_szamlazz_auto_completes')) && $this->is_invoice_generated($order_id)) {
							$return_info = $this->generate_invoice_complete($order_id);
						}
					}

					if($this->get_option('wc_szamlazz_defer') == 'yes') {
						self::$background_generator->save()->dispatch();
					}
				}

				//If there was an error while generating invoices automatically
				if($return_info && $return_info['error']) {
					$this->on_auto_invoice_error($order_id);
				}
			}
		}

	}

	//Autogenerate proform invoice
	public function on_order_processing( $order_id ) {

		//Only generate invoice, if it wasn't already generated & only if automatic invoice is enabled
		$order = new WC_Order($order_id);
		$payment_method = $this->get_order_property('payment_method',$order);

		if(get_post_meta( $order_id, '_wc_nyugta_type', true ) != 'receipt') {
			if($this->get_option('wc_szamlazz_payment_method_options')) {
				if($this->check_payment_method_options($payment_method, 'request')) {
					if(!$this->is_invoice_generated($order_id)) {
						if($this->get_option('wc_szamlazz_defer') == 'yes') {
							self::$background_generator->push_to_queue(
								array(
									'invoice_type' => 'proform',
									'order_id' => $order_id
								)
							);
							self::$background_generator->save()->dispatch();
						} else {
							$return_info = $this->generate_invoice($order_id,true);
						}
					}
				}
			} else {
				if($this->get_option('wc_szamlazz_payment_request_autos') && in_array($payment_method,$this->get_option('wc_szamlazz_payment_request_autos'))) {
					if(!$this->is_invoice_generated($order_id)) {
						$return_info = $this->generate_invoice($order_id,true);
					}
				}
			}
		}
	}

	//Autogenerate invoice
	public function on_order_deleted( $order_id ) {

		//Only generate sztornó, if regular invoice already generated & only if automatic invoice is enabled
		if($this->get_option('wc_szamlazz_auto') == 'yes') {

			if($this->is_invoice_generated($order_id)) {

				$return_info = false;

				//Check if we need to generate an invoice or a receipt
				if(get_post_meta( $order_id, '_wc_nyugta_type', true ) == 'receipt') {
					$return_info = $this->generate_receipt_sztorno($order_id);
				} else {
					$return_info = $this->generate_invoice_sztorno($order_id);
				}
			}
		}

	}


	//Send email on error
	public function on_auto_invoice_error( $order_id ) {
		update_option('_wc_szamlazz_error', $order_id);

		//Check if we need to send an email todo
		if($this->get_option('wc_szamlazz_error_email')) {
			$order = wc_get_order($order_id);
			$mailer = WC()->mailer();
			$content = wc_get_template_html( 'includes/emails/invoice-error.php', array(
				'order'         => $order,
				'email_heading' => __('Sikertelen számlakészítés', 'wc-szamlazz'),
				'plain_text'    => false,
				'email'         => $mailer,
				'sent_to_admin' => true,
			), '', plugin_dir_path( __FILE__ ) );
			$recipient = $this->get_option('wc_szamlazz_error_email');
			$subject = __("Sikertelen számlakészítés", 'wc-szamlazz');
			$headers = "Content-Type: text/html\r\n";
			$mailer->send( $recipient, $subject, $content, $headers );
		}

	}

	//Check if it was already generated or not
	public function is_invoice_generated( $order_id ) {
		$invoice_name = get_post_meta($order_id,'_wc_szamlazz',true);
		$invoice_own = get_post_meta($order_id,'_wc_szamlazz_own',true);
		if($invoice_name || $invoice_own) {
			return true;
		} else {
			return false;
		}
	}

	//Check if szállítólevél was already generated or not
	public function is_delivery_note_generated( $order_id ) {
		$invoice_name = get_post_meta($order_id,'_wc_szamlazz_szallitolevel',true);
		if($invoice_name) {
			return true;
		} else {
			return false;
		}
	}

	//Add icon to order list to show invoice
	public function add_listing_actions( $order ) {
		$order_id = $this->get_order_id($order);

		if($this->is_invoice_generated($order_id)):
		?>
			<a href="<?php echo $this->generate_download_link($order_id); ?>" class="button tips wc_szamlazz_button" target="_blank" alt="" data-tip="<?php _e('Szamlazz.hu számla','wc-szamlazz'); ?>">
				<img src="<?php echo WC_Szamlazz::$plugin_url . 'assets/images/invoice.svg'; ?>" alt="" width="12" height="16">
			</a>
		<?php
		endif;

		if(get_post_meta($order_id,'_wc_szamlazz_dijbekero_pdf',true)):
		?>
			<a href="<?php echo $this->generate_download_link($order_id,'dijbekero'); ?>" class="button tips wc_szamlazz_button" target="_blank" alt="" data-tip="<?php _e('Szamlazz.hu díjbekérő','wc-szamlazz'); ?>">
				<img src="<?php echo WC_Szamlazz::$plugin_url . 'assets/images/payment_request.svg'; ?>" alt="" width="12" height="16">
			</a>
		<?php
		endif;

		if(get_post_meta($order_id,'_wc_szamlazz_szallitolevel_pdf',true)):
		?>
			<a href="<?php echo $this->generate_download_link($order_id,'szallitolevel'); ?>" class="button tips wc_szamlazz_button" target="_blank" alt="" data-tip="<?php _e('Szamlazz.hu szállítólevél','wc-szamlazz'); ?>">
				<img src="<?php echo WC_Szamlazz::$plugin_url . 'assets/images/delivery_note.svg'; ?>" alt="" width="18" height="12">
			</a>
		<?php
		endif;
	}

	//Generate download url
	public function generate_download_link( $order_id, $type = false, $absolute = false) {
		if($order_id) {
			$pdf_name = '';
			if($type && $type == 'dijbekero') {
				$pdf_name = get_post_meta($order_id,'_wc_szamlazz_dijbekero_pdf',true);
			} else if($type && $type == 'sztorno') {
				$pdf_name = get_post_meta($order_id,'_wc_szamlazz_sztorno_pdf',true);
			} else if($type && $type == 'szallitolevel') {
				$pdf_name = get_post_meta($order_id,'_wc_szamlazz_szallitolevel_pdf',true);
			} else {
				$pdf_name = get_post_meta($order_id,'_wc_szamlazz_pdf',true);
			}

			if($pdf_name) {
				$UploadDir = wp_upload_dir();
				$UploadURL = $UploadDir['baseurl'];
				if($absolute) {
					$pdf_file_url = $UploadDir['basedir'].'/wc_szamlazz/'.$pdf_name;
				} else {
					$pdf_file_url = $UploadURL.'/wc_szamlazz/'.$pdf_name;
				}
				return $pdf_file_url;
			} else {
				return false;
			}
		} else {
			return false;
		}
	}

	//Get available checkout methods and ayment gateways
	public function get_available_payment_gateways() {
		$available_gateways = WC()->payment_gateways->payment_gateways();
		$available = array();
		$available['none'] = __('Válassz fizetési módot','wc-szamlazz');
		foreach ($available_gateways as $available_gateway) {
			$available[$available_gateway->id] = $available_gateway->title;
		}
		return $available;
	}

	//If the invoice is already generated without the plugin
	public function wc_szamlazz_already() {
		check_ajax_referer( 'wc_already_invoice', 'nonce' );
		if( true ) {
			if ( !current_user_can( 'edit_shop_orders' ) )  {
				wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
			}
			$orderid = intval($_POST['order']);
			$note = sanitize_text_field($_POST['note']);
			update_post_meta( $orderid, '_wc_szamlazz_own', $note );

			$response = array();
			$response['error'] = false;
			$response['messages'][] = __('Saját számla sikeresen hozzáadva.','wc-szamlazz');
			$response['invoice_name'] = $note;

			wp_send_json_success($response);
		}
	}

	//If the invoice is already generated without the plugin, turn it off
	public function wc_szamlazz_already_back() {
		check_ajax_referer( 'wc_already_invoice', 'nonce' );
		if( true ) {
			if ( !current_user_can( 'edit_shop_orders' ) )  {
				wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
			}
			$orderid = intval($_POST['order']);
			$note = sanitize_text_field($_POST['note']);
			update_post_meta( $orderid, '_wc_szamlazz_own', '' );

			$response = array();
			$response['error'] = false;
			$response['messages'][] = __('Visszakapcsolás sikeres.','wc-szamlazz');

			wp_send_json_success($response);
		}
	}

	//Add vat number field to checkout page
	public function add_vat_number_checkout_field($fields) {
		$show = true;

		if($this->get_option('wc_szamlazz_vat_number_form_min') == 'yes' && WC()->cart->get_taxes_total() < 100000) {
			$show = false;
		}

		if($show) {
			$fields['billing']['adoszam'] = array(
				 'label'     => __('Adószám', 'wc-szamlazz'),
				 'placeholder'   => _x('12345678-1-23', 'placeholder', 'wc-szamlazz'),
				 'required'  => false,
				 'class'     => array('form-row-wide'),
				 'clear'     => true,
				 'priority'	 => 35
			);
		}

		return $fields;
	}

	public function add_vat_number_info_notice($checkout) {
		if(WC()->cart->get_taxes_total() > 100000) {
			wc_print_notice( $this->get_option('wc_szamlazz_vat_number_notice'), 'notice' );
		}
	}

	public function save_vat_number( $order_id ) {
		if ( ! empty( $_POST['adoszam'] ) ) {
			$order = wc_get_order( $order_id );
			$order->update_meta_data( 'adoszam', sanitize_text_field( $_POST['adoszam'] ) );
			$adoszam_data = $this->get_vat_number_data(sanitize_text_field($_POST['adoszam']));
			if($adoszam_data) {
		    $order->update_meta_data( '_wc_szamlazz_adoszam_data', $adoszam_data );
			}
			$order->save();
		}
	}

	public function display_vat_number($order){
		$order_id = $this->get_order_id($order);
		if($adoszam = get_post_meta( $order_id, 'adoszam', true )) {
			echo '<p><strong>'.__('Adószám').':</strong> ' . $adoszam . '</p>';
		}
	}

	public function vat_number_validate($fields, $errors) {
		if($fields['adoszam']) {

			//Validate general format
			if(preg_match('/^(\d{7})(\d)\-([1-5])\-(0[2-9]|[13][0-9]|2[02-9]|4[0-4]|51)$/', sanitize_text_field($fields['adoszam']))) {

				//Check with the API too, but only if theres no more errors
				$error_codes = $errors->get_error_codes();
				if(empty( $error_codes )) {
					$adoszam_data = $this->get_vat_number_data(sanitize_text_field($fields['adoszam']));
					if($adoszam_data && !$adoszam_data['valid']) {
						$errors->add( 'validation', __( 'Az adószám nem megfelelő.', 'wc-szamlazz') );
					}
				}

			} else {
				$errors->add( 'validation', __( 'Az adószám formátuma nem megfelelő.', 'wc-szamlazz') );
			}

		}
	}

	//Generate Invoice with Ajax
	public function generate_invoice_complete_with_ajax() {
		check_ajax_referer( 'wc_generate_invoice', 'nonce' );
		if( true ) {
			$orderid = intval($_POST['order']);
			$return_info = $this->generate_invoice_complete($orderid);
			wp_send_json_success($return_info);
		}
	}

	//Generate XML for Szamla Agent
	public function generate_invoice_complete($orderId) {
		global $wpdb, $woocommerce;
		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlszamlakifiz xmlns="http://www.szamlazz.hu/xmlszamlakifiz" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlakifiz http://www.szamlazz.hu/docs/xsds/agentkifiz/xmlszamlakifiz.xsd"></xmlszamlakifiz>');

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());
		$szamla->beallitasok->addChild('szamlaszam', str_replace(array('.', ' ', "\n", "\t", "\r"), '', get_post_meta($orderId,'_wc_szamlazz',true)));
		$szamla->beallitasok->addChild('additiv', 'false');

		//Invoice details
		$kifizetes = $szamla->addChild('kifizetes');

		//Check if payment date is stored
		$date_paid = $order->get_date_paid();
		if( ! empty( $date_paid) ){
			$kifizetes->addChild('datum', $date_paid->date("Y-m-d") );
		} else {
			$kifizetes->addChild('datum', date('Y-m-d') );
		}
		$kifizetes->addChild('jogcim', $this->get_order_property('payment_method_title',$order) );
		$kifizetes->addChild('osszeg', round($order->get_total(),2));

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml_kifiz',$szamla,$order);
		$xml = $xml_szamla->asXML();

		//Get response from Számlázz.hu
		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_kifiz');

		if($xml_response['error']) {

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu teljesítettnek jelölés sikertelen! Agent hibakód: ', 'wc-szamlazz' ).urldecode($xml_response['agent_error']) );

			//Run action
			do_action('wc_szamlazz_after_invoice_error', $order, $xml_response);

			return $xml_response;

		} else {

			//Save data
			$xml_response['messages'][] = __('Jóváírás sikeresen rögzítve.','wc-szamlazz');

			//Store as a custom field
			update_post_meta( $orderId, '_wc_szamlazz_jovairas', time() );

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu jóváírás sikeresen rögzítve', 'wc-szamlazz' ) );

			//Text response
			$xml_response['link'] = '<p>'.__('Jóváírás rögzítve','wc-szamlazz').': '.date("Y-m-d").'</a></p>';

			return $xml_response;
		}

	}

	//Generate Sztornó invoice with Ajax
	public function generate_invoice_sztorno_with_ajax() {
		check_ajax_referer( 'wc_generate_invoice', 'nonce' );
		if( true ) {
			$orderid = intval($_POST['order']);
			$return_info = $this->generate_invoice_sztorno($orderid);
			wp_send_json_success($return_info);
		}
	}

	//Generate XML for Szamla Agent Sztornó
	public function generate_invoice_sztorno($orderId) {
		global $wpdb, $woocommerce;
		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlszamlast xmlns="http://www.szamlazz.hu/xmlszamlast" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlast http://www.szamlazz.hu/docs/xsds/agentst/xmlszamlast.xsd"></xmlszamlast>');

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());
		$electronic_invoice_type = $this->get_invoice_type($order);
		$szamla->beallitasok->addChild('eszamla', $electronic_invoice_type);
		$szamla->beallitasok->addChild('szamlaLetoltes', 'true');

		//Invoice details
		$fejlec = $szamla->addChild('fejlec');
		$fejlec->addChild('szamlaszam', str_replace(array('.', ' ', "\n", "\t", "\r"), '', get_post_meta($orderId,'_wc_szamlazz',true)));
		$fejlec->addChild('keltDatum', date('Y-m-d') );

		//Invoice details
		$elado = $szamla->addChild('elado');

		//Invoice details
		$vevo = $szamla->addChild('vevo');

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml_sztorno',$szamla,$order);
		$xml = $xml_szamla->asXML();

		//Get response from Számlázz.hu
		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_st');

		if($xml_response['error']) {

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu sztornó számlakészítés sikertelen! Agent hibakód: ', 'wc-szamlazz' ).urldecode($xml_response['agent_error']) );

			return $xml_response;
		} else {

			//Save data
			$xml_response['messages'][] = __('Sztornó számla létrehozva.','wc-szamlazz');


			//Get the Invoice ID from the response header
			$szlahu_szamlaszam = $this->xml_generator->get_invoice_id($xml_response['header_array']);

			//Download & Store PDF - generate a random file name so it will be downloadable later only by you
			$pdf_file_name = $this->xml_generator->save_pdf_file($orderId);

			//Store as a custom field
			update_post_meta( $orderId, '_wc_szamlazz_sztorno', $szlahu_szamlaszam );
			update_post_meta( $orderId, '_wc_szamlazz_sztorno_pdf', $pdf_file_name );

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu sztornó számla létrehozva: '.$szlahu_szamlaszam, 'wc-szamlazz' ) );

			//Return the download url
			$pdf_url = $this->generate_download_link($orderId,'sztorno');

			//Remove existing szamla
			delete_post_meta( $orderId, '_wc_szamlazz' );
			delete_post_meta( $orderId, '_wc_szamlazz_pdf' );

			$xml_response['link'] = '<p>'.__('Sztornó számla','wc-szamlazz').': '.date("Y-m-d").'</a></p>';

			$xml_response['link'] = '<p>Sztornó számla létrehozva: <a href="'.$pdf_url.'" target="_blank">'.__('Megtekintése','wc-szamlazz').'</a><br><small>Az oldal frissítése után van lehetőség új számlát készíteni</small></p>';

			return $xml_response;
		}

	}

	public function wc_szamlazz_ipn_process() {
		if (isset($_GET['wc_szamlazz_ipn_url'])) {
			if($_GET['wc_szamlazz_ipn_url'] != get_option( 'wc_szamlazz_ipn_url') || empty($_GET['wc_szamlazz_ipn_url'])) {
				return false;
			}
			$order = new WC_Order($_POST['szlahu_rendelesszam']);
			do_action('wc_szamlazz_before_ipn_process', $order);

			$orderId = $this->get_order_id($order);
			$order->add_order_note( __( 'Szamlazz.hu jóváírás sikeresen rögzítve(IPN alapján)', 'wc-szamlazz' ) );
			update_post_meta( $_POST['szlahu_rendelesszam'], '_wc_szamlazz_jovairas', time() );

			//Check if we don't have an invoice, but IPN sent one. If so, download the PDF invoice and store it in WooCommerce
			if(!$this->is_invoice_generated($this->get_order_id($order)) && !empty($_POST['szlahu_szamlaszam'])) {
				$szamlaszam = $_POST['szlahu_szamlaszam'];

				//Build Xml
				$szamla = new SimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlszamlapdf xmlns="http://www.szamlazz.hu/xmlszamlapdf" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlszamlapdf xmlszamlapdf.xsd"></xmlszamlapdf>');

				//Account & Invoice settings
				$szamla->addChild('felhasznalo', $this->get_option('wc_szamlazz_username'));
				$szamla->addChild('jelszo', $this->get_option('wc_szamlazz_password'));
				$szamla->addChild('szamlaszam', $szamlaszam);
				$szamla->addChild('valaszVerzio', '1');

				//Generate XML
				$xml = $szamla->asXML();
				$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_pdf');
				if(!$xml_response['error']) {

					//Download & Store PDF - generate a random file name so it will be downloadable later only by you
					$pdf_file_name = $this->xml_generator->save_pdf_file($orderId);

					//Store as a custom field
					update_post_meta( $orderId, '_wc_szamlazz', $szamlaszam );

					//Update order notes
					$order->add_order_note( __( 'Szamlazz.hu számla sikeresen létrehozva, IPN-en keresztül. A számla sorszáma: ', 'wc-szamlazz' ).$szamlaszam );

					//Store the filename
					update_post_meta( $orderId, '_wc_szamlazz_pdf', $pdf_file_name );
				}
			}

			//Update order status to complete if needed
			if($this->get_option('wc_szamlazz_ipn_close_order') == 'yes') {
				$order->update_status( 'completed' );
			}

			do_action('wc_szamlazz_after_ipn_process', $order);

			exit();
		}
	}

	//Get order ID(backward compatibility), for WC3.0+
	public function get_order_id($order) {
		$id = method_exists( $order, 'get_id' ) ? $order->get_id() : $order->id;
		return $id;
	}

	//Get order details(backward compatiblity), for WC3.0+
	public function get_order_property($property,$order) {

		//3.0+
		$value = '';
		if(method_exists( $order, 'get_id' )) {
			$property = 'get_'.$property;
			$value = $order->$property();
		} else {
			$value = $order->$property;
		}

		return $value;

	}

	//Add download icons to order details page
	public function szamlazz_download_button($actions, $order) {
		$order_id = $this->get_order_id($order);
		if($this->get_option('wc_szamlazz_customer_download','no') == 'yes') {

			//Add invoice link
			if(get_post_meta($order_id,'_wc_szamlazz_pdf',true)) {
				$link = $this->generate_download_link($order_id);
				$actions['szamlazz_pdf'] = array(
					'url'  => $link,
					'name' => __( 'Számla', 'wc_szamlazz' )
				);
			}

			//Add payment request link
			if(get_post_meta($order_id,'_wc_szamlazz_dijbekero_pdf',true)) {
				$link_request = $this->generate_download_link($order_id,'dijbekero');
				$actions['szamlazz_pdf'] = array(
					'url'  => $link_request,
					'name' => __( 'Díjbekérő', 'wc_szamlazz' )
				);
			}
		}
		return $actions;
	}

	//Get options stored
	public function get_option($key, $default = '') {
		$settings = get_option( 'woocommerce_wc_szamlazz_settings', null );
		$value = $default;

		if($settings && isset($settings[$key])) {
			$value = $settings[$key];
		} else if(get_option($key)) {
			$value = get_option($key);
		}

		//Try to get password from wp-config
		if($key == 'wc_szamlazz_password' && defined( 'WC_SZAMLAZZ_JELSZO' )) {
			$value = WC_SZAMLAZZ_JELSZO;
		}

		if($key == 'wc_szamlazz_api_key' && defined( 'WC_SZAMLAZZ_AGENT_KULCS' )) {
			$value = WC_SZAMLAZZ_AGENT_KULCS;
		}

		return $value;
	}

	public function store_nyugta_session_data() {
		//Set session data, if not set previously
		if(!WC()->session->wc_szamlazz_nyugta) {
			WC()->session->set( 'wc_szamlazz_nyugta', 'receipt' );
		}
	}

	public function nyugta_get_checkout_value($value, $input) {
		if($input == 'nyugta') {
			$nyugta = WC()->session->wc_szamlazz_nyugta;
			if($nyugta == 'invoice') {
				$value = true;
			} else {
				$value = false;
			}
		}
		return $value;
	}

	//Add E-Nyugta checkbox to checkout
	public function add_nyugta_checkout_field($fields) {

		$fields['billing']['nyugta'] = array(
			'type'	=> 'checkbox',
			'label'	=> __('Nyugta helyett szeretnék számlát kérni', 'wc-szamlazz'),
			'priority' => 0
		);

		//Hide billing fields if we don't need an invoice, just a receipt
		if(WC()->session->wc_szamlazz_nyugta &&  WC()->session->wc_szamlazz_nyugta == 'receipt') {
	    unset($fields['billing']['billing_company']);
	    unset($fields['billing']['billing_address_1']);
	    unset($fields['billing']['billing_address_2']);
	    unset($fields['billing']['billing_city']);
	    unset($fields['billing']['billing_postcode']);
	    unset($fields['billing']['billing_country']);
	    unset($fields['billing']['billing_state']);
	    unset($fields['billing']['billing_phone']);
	    unset($fields['billing']['billing_address_2']);
	    unset($fields['billing']['billing_postcode']);
	    unset($fields['billing']['billing_company']);
	    unset($fields['billing']['billing_city']);
			unset($fields['order']['order_comments']);
		}

		//Load temporary values for name and email, which was saved when receipt or invoice was switched on checkout
		$tempCustomer = WC()->session->wc_szamlazz_temp_customer;
		if($tempCustomer) {
			$storedFields = array('billing_first_name', 'billing_last_name', 'billing_email');
			foreach ($storedFields as $storedField) {
				if(isset(WC()->session->wc_szamlazz_temp_customer[$storedField])) {
					$fields['billing'][$storedField]['default'] = WC()->session->wc_szamlazz_temp_customer[$storedField];
				}
			}
		}

		//Move email field below names
		$fields['billing']['billing_email']['priority'] = 21;

		return $fields;
	}

	public function wc_szamlazz_nyugta_check() {
		check_ajax_referer( 'update-order-review', 'nonce' );

		if($_POST['checked'] == 'invoice') {
			WC()->session->set( 'wc_szamlazz_nyugta', 'invoice' );
		} else {
			WC()->session->set( 'wc_szamlazz_nyugta', 'receipt' );
		}

		//Update name and email, so if it was filled already, it will stay filled after a reload
		$customer = array();

		if ( ! empty( $_POST['billing_first_name'] ) ) {
			$customer['billing_first_name'] = $_POST['billing_first_name'];
		}

		if ( ! empty( $_POST['billing_last_name'] ) ) {
			$customer['billing_last_name'] = $_POST['billing_last_name'];
		}

		if ( ! empty( $_POST['billing_email'] ) ) {
			$customer['billing_email'] = $_POST['billing_email'];
		}

		WC()->session->set( 'wc_szamlazz_temp_customer', $customer);

		wp_send_json_success();
	}

	public function save_nyugta_session_data( $order_id ) {
		if ( ! empty( WC()->session->wc_szamlazz_nyugta ) ) {
			update_post_meta( $order_id, '_wc_nyugta_type', sanitize_text_field( WC()->session->wc_szamlazz_nyugta ) );
		}
	}

	//Generate Invoice with Ajax
	public function generate_receipt_with_ajax() {
		check_ajax_referer( 'wc_generate_invoice', 'nonce' );
		if( true ) {
			$orderid = intval($_POST['order']);
			$return_info = $this->generate_receipt($orderid);
			wp_send_json_success($return_info);
		}
	}

	//Generate XML for Szamla Agent
	public function generate_receipt($orderId) {
		global $wpdb, $woocommerce;
		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlnyugtacreate xmlns="http://www.szamlazz.hu/xmlnyugtacreate" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtacreate http://www.szamlazz.hu/docs/xsds/nyugta/xmlnyugtacreate.xsd"></xmlnyugtacreate>');

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());
		$szamla->beallitasok->addChild('pdfLetoltes', 'true');

		//Invoice details
		$fejlec = $szamla->addChild('fejlec');
		$fejlec->addChild('hivasAzonosito', $order->get_order_number());
		$fejlec->addChild('elotag', $this->get_option('wc_szamlazz_nyugta_elotag'));
		$fejlec->addChild('fizmod', $this->get_order_property('payment_method_title',$order));
		$order_currency = method_exists( $order, 'get_currency' ) ? $order->get_currency() : $order->get_order_currency();
		$fejlec->addChild('penznem', $order_currency);
		$fejlec->addChild('megjegyzes', $this->get_option('wc_szamlazz_nyugta_megjegyzes'));

		if($order_currency != 'HUF') {
			//if the base currency is not HUF, we should define currency rates
			$fejlec->addChild('devizabank', 'MNB');
			$exchange_rate = get_transient( 'wc_szamlazz_mnb_arfolyam_kozep' );
			if(!$exchange_rate) {
				$exchange_rate = wp_remote_retrieve_body( wp_remote_get( 'http://api.napiarfolyam.hu?bank=mnb&valuta='.$order_currency ) );
				$napiarfolyam_xml = new SimpleXMLElement($exchange_rate);
				$napiarfolyam_kozep = (Array)$napiarfolyam_xml->deviza->item->kozep;
				$napiarfolyam_kozep = $napiarfolyam_kozep[0];
				set_transient( 'wc_szamlazz_mnb_arfolyam_kozep', $napiarfolyam_kozep, 60*60*12 );
				$exchange_rate = $napiarfolyam_kozep;
			}
			$fejlec->addChild('devizaarf', $exchange_rate);
		}

		//Order Items
		$tetelek = $szamla->addChild('tetelek');
		foreach( $order_items as $termek ) {

			//This is for 3.0+
			$product_id = $termek['product_id'];
			$sku = $termek->get_product()->get_sku();
			$tetel = $tetelek->addChild('tetel');
			if($termek->get_product() && $termek->get_product()->get_meta('wc_szamlazz_tetel_nev') && $termek->get_product()->get_meta('wc_szamlazz_tetel_nev') != '') {
				$tetel->addChild('megnevezes', htmlspecialchars($termek->get_product()->get_meta('wc_szamlazz_tetel_nev')));
			} else {
				$tetel->addChild('megnevezes',htmlspecialchars($termek->get_name()));
			}
			$tetel->addChild('azonosito',$sku);
			$tetel->addChild('mennyiseg',$termek->get_quantity());
			$tetel->addChild('mennyisegiEgyseg','');
			if(round($termek->get_total(),2) == 0) {
				$tetel->addChild('nettoEgysegar',0);
				$tetel->addChild('afakulcs',0);
			} else {
				$tetel->addChild('nettoEgysegar', round($termek->get_total(),2)/$termek->get_quantity());
				$tetel->addChild('afakulcs', round( ($termek->get_total_tax()/$termek->get_total()) * 100 ) );
			}
			$tetel->addChild('netto', round($termek->get_total(),2));
			$tetel->addChild('afa', round($termek->get_total_tax(),2));
			$tetel->addChild('brutto', round($termek->get_total(),2)+round($termek->get_total_tax(),2));

		}

		//Shipping
		if($order->get_shipping_methods()) {
			$tetel = $tetelek->addChild('tetel');
			$tetel->addChild('megnevezes', htmlspecialchars($order->get_shipping_method()));
			$tetel->addChild('mennyiseg','1');
			$tetel->addChild('mennyisegiEgyseg','');
			$order_shipping = method_exists( $order, 'get_shipping_total' ) ? $order->get_shipping_total() : $order->order_shipping;
			$order_shipping_tax = method_exists( $order, 'get_shipping_tax' ) ? $order->get_shipping_tax() : $order->order_shipping_tax;
			$tetel->addChild('nettoEgysegar',round($order_shipping,2));
			if($order_shipping == 0) {
				$tetel->addChild('afakulcs','0');
			} else {
				$tetel->addChild('afakulcs',round(($order_shipping_tax/$order_shipping)*100));
			}
			$tetel->addChild('netto',round($order_shipping,2));
			$tetel->addChild('afa',round($order_shipping_tax,2));
			$tetel->addChild('brutto',round($order_shipping,2)+round($order_shipping_tax,2));
		}

		//Extra Fees
		$fees = $order->get_fees();
		if(!empty($fees)) {
			foreach( $fees as $fee ) {
				$tetel = $tetelek->addChild('tetel');
				$tetel->addChild('megnevezes',htmlspecialchars($fee["name"]));
				$tetel->addChild('mennyiseg',1);
				$tetel->addChild('mennyisegiEgyseg','');
				$tetel->addChild('nettoEgysegar',round($fee["line_total"],2));
				$tetel->addChild('afakulcs',round(($fee["line_tax"]/$fee["line_total"])*100));
				$tetel->addChild('netto',round($fee["line_total"],2));
				$tetel->addChild('afa',round($fee["line_tax"],2));
				$tetel->addChild('brutto',round($fee["line_total"],2)+round($fee["line_tax"],2));
			}
		}

		//Change afakulcs if needed
		if($this->get_option('wc_szamlazz_afakulcs')) {
			foreach($tetelek->tetel as $tetel) {
				$tetel->afakulcs = $this->get_option('wc_szamlazz_afakulcs');
			}
		}

		//Discount
		$order_discount = method_exists( $order, 'get_discount_total' ) ? $order->get_discount_total() : $order->order_discount;
		$discout_details = $this->get_coupon_invoice_item_details($order);
		if(method_exists( $order, 'get_id' )) {
			if ( $order_discount > 0 ) {
				if($szamla->fejlec->megjegyzes) {
					$szamla->fejlec->megjegyzes .= ' ';
				}
				$szamla->fejlec->megjegyzes .= $discout_details['desc'];
			}
		} else {
			$order_discount = method_exists( $order, 'get_discount_total' ) ? $order->get_discount_total() : $order->order_discount;
			if ( $order_discount > 0 ) {
				$tetel = $tetelek->addChild('tetel');
				$tetel->addChild('megnevezes',$discout_details['title']);
				$tetel->addChild('mennyiseg','1');
				$tetel->addChild('mennyisegiEgyseg','');
				$tetel->addChild('nettoEgysegar',-$order_discount);
				$tetel->addChild('afakulcs',0);
				$tetel->addChild('netto',-$order_discount);
				$tetel->addChild('afa',0);
				$tetel->addChild('brutto',-$order_discount);
			}
		}

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml_receipt',$szamla,$order);
		$xml = $xml_szamla->asXML();

		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_nyugta_create');
		$agent_body_xml = new SimpleXMLElement($xml_response['agent_body']);

		// ezt majd true-ra állítjuk ha volt hiba
		$volt_hiba = false;

		// ebben lesznek a hiba információk, plusz a bodyban
		$agent_error = '';
		$agent_error_code = '';

		// Nézzük meg volt e hiba
		if(!filter_var($agent_body_xml->sikeres, FILTER_VALIDATE_BOOLEAN)) {
			$volt_hiba = true;
			$agent_error = $agent_body_xml->hibauzenet;
			$agent_error_code = $agent_body_xml->hibakod;
		}

		if($xml_response['error'] || $volt_hiba) {
			$xml_response['error'] = true;
			$xml_response['messages'][] = 'Agent hibakód: '.$agent_error_code;
			$xml_response['messages'][] = 'Agent hibaüzenet: '.$agent_error;
			$order->add_order_note( __( 'Szamlazz.hu nyugta készítés sikertelen! Agent hibakód: ', 'wc-szamlazz' ).urldecode($agent_error) );
			do_action('wc_szamlazz_after_receipt_error', $order, $xml_response, $agent_error_code, $agent_error, $agent_body_xml);
			return $xml_response;
		} else {

			//Get the Invoice ID from the response header
			$szlahu_szamlaszam = (string)$agent_body_xml->nyugta->alap->nyugtaszam;

			//Download & Store PDF - generate a random file name so it will be downloadable later only by you
			$pdf_content = base64_decode($agent_body_xml->nyugtaPdf);
			$pdf_file_name = $this->xml_generator->save_pdf_file($orderId, $pdf_content);

			//Create response
			$xml_response['invoice_name'] = $szlahu_szamlaszam;

			//We send an email too?
			$auto_email = $this->get_option('wc_szamlazz_nyugta_email');

			if($auto_email) {

				$email_info = $this->send_receipt($orderId, $order, $szlahu_szamlaszam);

				if($email_info['error']) {
					$xml_response['error'] = true;
					$xml_response['messages'] = $email_info['messages'];
					$xml_response['messages'][] = __('Nyugta sikeresen létrehozva, de emailben nem sikerült elküldeni.','wc-szamlazz');
				} else {
					$xml_response['messages'][] = __('Nyugta sikeresen létrehozva és elküldve a vásárlónak emailben.','wc-szamlazz');
				}

			} else {
				$xml_response['messages'][] = __('Nyugta sikeresen létrehozva.','wc-szamlazz');
			}

			//Store as a custom field
			update_post_meta( $orderId, '_wc_szamlazz', $szlahu_szamlaszam );

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu Nyugta sikeresen létrehozva. A Nyugta sorszáma: ', 'wc-szamlazz' ).$szlahu_szamlaszam );

			//Store the filename
			update_post_meta( $orderId, '_wc_szamlazz_pdf', $pdf_file_name );

			//Return the download url
			$pdf_url = $this->generate_download_link($orderId);

			$xml_response['link'] = '<p><a href="'.$pdf_url.'" id="wc_szamlazz_download" class="button button-primary" target="_blank">'.__('Nyugta megtekintése','wc-szamlazz').'</a></p>';

			do_action('wc_szamlazz_after_receipt_success', $order, $xml_response, $szlahu_szamlaszam, $pdf_url);
			return $xml_response;
		}

	}

	//Generate XML for Szamla Agent
	public function send_receipt($orderId, $order, $receiptID) {
		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlnyugtasend xmlns="http://www.szamlazz.hu/xmlnyugtasend" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtasend http://www.szamlazz.hu/docs/xsds/nyugtasend/xmlnyugtasend.xsd"></xmlnyugtasend>');

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());

		//Invoice details
		$fejlec = $szamla->addChild('fejlec');
		$fejlec->addChild('nyugtaszam', $receiptID);

		//Email details
		$email = $szamla->addChild('emailKuldes');
		$email->addChild('email', $this->get_order_property('billing_email', $order));

		if($this->get_option('wc_szamlazz_nyugta_email_replyto')) {
			$email->addChild('emailReplyto', $this->get_option('wc_szamlazz_nyugta_email_replyto'));
		}

		if($this->get_option('wc_szamlazz_nyugta_email_subject')) {
			$email->addChild('emailTargy', $this->get_option('wc_szamlazz_nyugta_email_subject'));
		}

		if($this->get_option('wc_szamlazz_nyugta_email_text')) {
			$email->addChild('emailSzoveg', $this->get_option('wc_szamlazz_nyugta_email_text'));
		}

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml_receipt_send', $szamla, $order);
		$xml = $xml_szamla->asXML();
		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_nyugta_send');
		$agent_body_xml = new SimpleXMLElement($xml_response['agent_body']);

		// ezt majd true-ra állítjuk ha volt hiba
		$volt_hiba = false;

		// ebben lesznek a hiba információk, plusz a bodyban
		$agent_error = '';
		$agent_error_code = '';

		// Nézzük meg volt e hiba
		if(!filter_var($agent_body_xml->sikeres, FILTER_VALIDATE_BOOLEAN)) {
			$volt_hiba = true;
			$agent_error = $agent_body_xml->hibauzenet;
			$agent_error_code = $agent_body_xml->hibakod;
		}

		if ($volt_hiba) {
			$xml_response['error'] = true;

			// ha a számla nem készült el kiírjuk amit lehet
			$xml_response['messages'][] = 'Agent hibakód: '.$agent_error_code;
			$xml_response['messages'][] = 'Agent hibaüzenet: '.$agent_error;

			do_action('wc_szamlazz_after_receipt_send_error', $order, $xml_response, $agent_error_code, $agent_error, $agent_body_xml);

			// dobunk egy kivételt
			return $xml_response;

		} else {

			do_action('wc_szamlazz_after_receipt_send_success', $order, $xml_response);
			return $xml_response;

		}

	}

	//Generate Sztornó invoice with Ajax
	public function generate_receipt_sztorno_with_ajax() {
		check_ajax_referer( 'wc_generate_invoice', 'nonce' );
		if( true ) {
			$orderid = intval($_POST['order']);
			$return_info = $this->generate_receipt_sztorno($orderid);

			wp_send_json_success($return_info);
		}
	}

	//Generate XML for Szamla Agent Sztornó
	public function generate_receipt_sztorno($orderId) {
		global $wpdb, $woocommerce;
		$order = new WC_Order($orderId);
		$order_items = $order->get_items();

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmlnyugtast xmlns="http://www.szamlazz.hu/xmlnyugtast" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmlnyugtast http://www.szamlazz.hu/docs/xsds/xmlnyugtast/xmlnyugtast.xsd"></xmlnyugtast>');

		//Account & Invoice settings
		$szamla->appendXML($this->get_authentication_xml_object());
		$szamla->beallitasok->addChild('pdfLetoltes', 'true');

		//Invoice details
		$fejlec = $szamla->addChild('fejlec');
		$fejlec->addChild('nyugtaszam', str_replace(array('.', ' ', "\n", "\t", "\r"), '', get_post_meta($orderId,'_wc_szamlazz',true)));

		//Generate XML
		$xml_szamla = apply_filters('wc_szamlazz_xml_sztorno_nyugta',$szamla,$order);
		$xml = $xml_szamla->asXML();
		$xml_response = $this->xml_generator->generate($xml, $orderId, 'action-szamla_agent_nyugta_storno');
		$agent_body_xml = new SimpleXMLElement($xml_response['agent_body']);

		// ezt majd true-ra állítjuk ha volt hiba
		$volt_hiba = false;

		// ebben lesznek a hiba információk, plusz a bodyban
		$agent_error = '';
		$agent_error_code = '';

		// Nézzük meg volt e hiba
		if(!filter_var($agent_body_xml->sikeres, FILTER_VALIDATE_BOOLEAN)) {
			$volt_hiba = true;
			$agent_error = $agent_body_xml->hibauzenet;
			$agent_error_code = $agent_body_xml->hibakod;
		}

		if ($volt_hiba || $xml_response['error']) {
			$xml_response['error'] = true;

			// ha a számla nem készült el kiírjuk amit lehet
			$xml_response['messages'][] = 'Agent hibakód: '.$agent_error_code;
			$xml_response['messages'][] = 'Agent hibaüzenet: '.$agent_error;

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu nyugta készítés sikertelen! Agent hibakód: ', 'wc-szamlazz' ).urldecode($agent_error) );

			do_action('wc_szamlazz_after_invoice_error', $order, $xml_response, $agent_error_code, $agent_error, $agent_body_xml);

			// dobunk egy kivételt
			return $xml_response;

		} else {

			//Save data
			$xml_response['messages'][] = __('Sztornó nyugta létrehozva.','wc-szamlazz');

			//Get the Invoice ID from the response header
			$szlahu_szamlaszam = (string)$agent_body_xml->nyugta->alap->nyugtaszam;
			$xml_response['invoice_name'] = $szlahu_szamlaszam;

			//Download & Store PDF - generate a random file name so it will be downloadable later only by you
			$pdf_content = base64_decode($agent_body_xml->nyugtaPdf);
			$pdf_file_name = $this->xml_generator->save_pdf_file($orderId, $pdf_content);

			//Store as a custom field
			update_post_meta( $orderId, '_wc_szamlazz_sztorno', $szlahu_szamlaszam );
			update_post_meta( $orderId, '_wc_szamlazz_sztorno_pdf', $pdf_file_name );

			//Update order notes
			$order->add_order_note( __( 'Szamlazz.hu sztornó nyugta létrehozva: '.$szlahu_szamlaszam, 'wc-szamlazz' ) );

			//Return the download url
			$pdf_url = $this->generate_download_link($orderId,'sztorno');

			//Remove existing szamla
			delete_post_meta( $orderId, '_wc_szamlazz' );
			delete_post_meta( $orderId, '_wc_szamlazz_pdf' );
			delete_post_meta( $orderId, '_wc_nyugta_type');

			$xml_response['link'] = '<p>'.__('Sztornó nyugta','wc-szamlazz').': '.date("Y-m-d").'</a></p>';

			$xml_response['link'] = '<p>Sztornó nyugta létrehozva: <a href="'.$pdf_url.'" target="_blank">'.__('Megtekintés','wc-szamlazz').'</a><br><small>Az oldal frissítése után van lehetőség új nyugtát készíteni</small></p>';

			return $xml_response;
		}

	}

	//Email attachment
	public function email_attachment($attachments, $email_id, $order){
		$order_id = method_exists( $order, 'get_id' ) ? $order->get_id() : $order->id;
		if( $email_id === 'customer_completed_order' || $email_id === 'customer_invoice'){
			$pdf_url = $this->generate_download_link($order_id, false, true);
			if($pdf_url) $attachments[] = $pdf_url;
		}

		if( $email_id === 'customer_processing_order' || $email_id === 'customer_on_hold_order') {
			$pdf_url = $this->generate_download_link($order_id, 'dijbekero', true);
			if($pdf_url) $attachments[] = $pdf_url;
		}

		if( $email_id === 'customer_refunded_order' || $email_id === 'cancelled_order') {
			$pdf_url = $this->generate_download_link($order_id, 'sztorno', true);
			if($pdf_url) $attachments[] = $pdf_url;
		}
		return $attachments;
	}

	//Plugin links
	public function plugin_action_links( $links ) {
		$action_links = array(
			'settings' => '<a href="' . admin_url( 'admin.php?page=wc-settings&tab=integration&section=wc_szamlazz' ) . '" aria-label="' . esc_attr__( 'Számlázz.hu Beállítások', 'wc-szamlazz' ) . '">' . esc_html__( 'Beállítások', 'wc-szamlazz' ) . '</a>',
			'documentation' => '<a href="https://szamlazz.visztpeter.me/dokumentacio/" target="_blank" aria-label="' . esc_attr__( 'Számlázz.hu Beállítások', 'wc-szamlazz' ) . '">' . esc_html__( 'Dokumentáció', 'wc-szamlazz' ) . '</a>'
		);

		if (!get_option('_wc_szamlazz_pro_enabled') ) {
			$action_links['get-pro'] = '<a target="_blank" rel="noopener noreferrer" style="color:#46b450;" href="https://szamlazz.visztpeter.me/" aria-label="' . esc_attr__( 'Számlázz.hu Pro verzió', 'wc-szamlazz' ) . '">' . esc_html__( 'Pro verzió', 'wc-szamlazz' ) . '</a>';
		}
		return array_merge( $action_links, $links );
	}

	public function get_invoice_type($order) {
		$electronic_invoice_type = 'false';
		if($this->get_option('wc_szamlazz_invoice_type') != 'paper') {
			$electronic_invoice_type = 'true';
		}

		if($this->get_order_property('billing_company',$order)) {
			if($this->get_option('wc_szamlazz_invoice_type_company') == 'electronic') {
				$electronic_invoice_type = 'true';
			} elseif ($this->get_option('wc_szamlazz_invoice_type_company') == 'paper') {
				$electronic_invoice_type = 'false';
			}
		}

		return $electronic_invoice_type;
	}

	public function check_payment_method_options($payment_method_id, $option) {
		$found = false;
		$payment_method_options = $this->get_option('wc_szamlazz_payment_method_options');
		if(isset($payment_method_options[$payment_method_id]) && isset($payment_method_options[$payment_method_id][$option])) {
			$found = $payment_method_options[$payment_method_id][$option];
		}
		return $found;
	}

	public function get_payment_method_deadline($payment_method_id) {
		$deadline = $this->get_option('wc_szamlazz_payment_deadline');
		$payment_method_options = $this->get_option('wc_szamlazz_payment_method_options');
		if($payment_method_options && isset($payment_method_options[$payment_method_id]) && isset($payment_method_options[$payment_method_id]['deadline'])) {
			$deadline = $payment_method_options[$payment_method_id]['deadline'];
		}
		return $deadline;
	}

	public function is_order_hungarian($order) {
		return $order->get_billing_country() == 'HU';
	}

	public function get_accounting_details($order, $category) {
		$function_enabled = $this->get_option('wc_szamlazz_accounting_details_enabled');
		$saved_values = get_option('wc_szamlazz_accounting_details');
		if(!$saved_values || !$function_enabled) return false;
		if(is_array($category) && count($category) < 1) return false;

		if(is_array($category)) {
			$category_id = esc_attr( $category[0]->term_id );
		} else {
			$category_id = esc_attr( $category );
		}

		$detail_items = array('afa_fokonyvi_szam', 'fokonyvi_szam', 'gazd_esem', 'afa_gazd_esem');
		$is_hungarian = $this->is_order_hungarian($order);
		$suffix = ($is_hungarian) ? '_hu' : '_kulfold';
		$defaults = array();
		$data = array();

		foreach ($detail_items as $detail_item) {
			$defaults[$detail_item] = esc_attr( $saved_values['default'][$detail_item.$suffix]);
			if(isset($category->term_id) && $category->term_id && $saved_values[esc_attr( $category->term_id )][$detail_item.$suffix] && $saved_values[esc_attr( $category->term_id )][$detail_item.$suffix] != '') {
				$data[$detail_item] = $saved_values[esc_attr( $category->term_id )][$detail_item.$suffix];
			}
		}

		$data = wp_parse_args( $data, $defaults );
		return $data;
	}

	public function get_authentication_xml_object() {
		$beallitasok = new WCSzamlazzSimpleXMLElement('<beallitasok></beallitasok>');
		if($this->get_option('wc_szamlazz_api_key')) {
			$beallitasok->addChild('szamlaagentkulcs', $this->get_option('wc_szamlazz_api_key'));
		} else {
			$beallitasok->addChild('felhasznalo', $this->get_option('wc_szamlazz_username'));
			$beallitasok->addChild('jelszo', $this->get_option('wc_szamlazz_password'));
		}
		return $beallitasok;
	}

	public function get_order_item_tax_label($order, $item) {
		$tax_item_label = '';

		if(wc_tax_enabled()) {
			$tax_items_labels = array();
			$valid_tax_labels = array('TAM', 'AAM', 'EU', 'EUK', 'MAA', 'F.AFA', 'ÁKK');

			//Get all tax labels indexed by rate id
			foreach ( $order->get_items('tax') as $tax_item ) {
				$tax_items_labels[$tax_item->get_rate_id()] = $tax_item->get_label();
			}

			//Get line item tax id and find label
			if(count($tax_items_labels) > 0) {
				$taxes = $item->get_taxes();
				foreach( $taxes['subtotal'] as $rate_id => $tax ){
					$tax_item_label = $tax_items_labels[$rate_id];
				}
			}

			//If its not a valid label
			if(!in_array($tax_item_label, $valid_tax_labels)) {
				$tax_item_label = '';
			}
		}

		//If theres no ID, return percentage value
		if($tax_item_label == '') {
			if($this->get_option('wc_szamlazz_coupon_type') == 'yes') {
				$tax_item_label = round( ($item->get_subtotal_tax()/$item->get_subtotal()) * 100 );
			} else {
				$tax_item_label = round( ($item->get_total_tax()/$item->get_total()) * 100 );
			}
		}

		return $tax_item_label;
	}

	public function get_order_shipping_tax_label($order, $shipping_item_obj) {
		$tax_item_label = '';

		if(wc_tax_enabled()) {
			$valid_tax_labels = array('TAM', 'AAM', 'EU', 'EUK', 'MAA', 'F.AFA', 'ÁKK');

			$tax_data = $shipping_item_obj->get_taxes();
			foreach ( $order->get_items('tax') as $tax_item ) {
				$tax_item_id    = $tax_item->get_rate_id();
				$tax_item_total = isset( $tax_data['total'][ $tax_item_id ] ) ? $tax_data['total'][ $tax_item_id ] : '';

				if($tax_item_total != '') {
					$tax_item_label = $tax_item->get_label();
				}
			}

			//If its not a valid label
			if(!in_array($tax_item_label, $valid_tax_labels)) {
				$tax_item_label = '';
			}
		}

		if($tax_item_label == '') {
			$order_shipping = $shipping_item_obj->get_total();
			$order_shipping_tax = $shipping_item_obj->get_total_tax();
			$tax_item_label = round(($order_shipping_tax/$order_shipping)*100);
		}

		return $tax_item_label;
	}

	public function get_vat_number_data($vat_number) {

		//Build Xml
		$szamla = new WCSzamlazzSimpleXMLElement('<?xml version="1.0" encoding="UTF-8"?><xmltaxpayer xmlns="http://www.szamlazz.hu/xmltaxpayer" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.szamlazz.hu/xmltaxpayer http://www.szamlazz.hu/docs/xsds/agent/xmltaxpayer.xsd"></xmltaxpayer>');
		$szamla->appendXML($this->get_authentication_xml_object());
		$szamla->addChild('torzsszam', mb_substr($vat_number, 0, 8));
		$xml = $szamla->asXML();

		//Get response from Számlázz.hu
		$xml_response = $this->xml_generator->generate($xml, rand(), 'action-szamla_agent_taxpayer');
		if($xml_response['error']) {
			return false;
		} else {
			$agent_body_xml = simplexml_load_string($xml_response['agent_body']);
			$json = json_encode($agent_body_xml);
			$array = json_decode($json,TRUE);
			$response = array(
				"valid" => $array['taxpayerValidity'],
				"name" => $array['taxpayerData']['taxpayerName'],
				"address" => array(
					"countryCode" => $array['taxpayerData']['taxpayerAddress']['countryCode'],
					"postalCode" => $array['taxpayerData']['taxpayerAddress']['postalCode'],
					"city" => $array['taxpayerData']['taxpayerAddress']['city'],
					"streetName" => $array['taxpayerData']['taxpayerAddress']['streetName'],
					"publicPlaceCategory" => $array['taxpayerData']['taxpayerAddress']['publicPlaceCategory'],
					"number" => $array['taxpayerData']['taxpayerAddress']['number'],
					"building" => $array['taxpayerData']['taxpayerAddress']['building'],
					"staircase" => $array['taxpayerData']['taxpayerAddress']['staircase'],
					"floor" => $array['taxpayerData']['taxpayerAddress']['floor'],
					"door" => $array['taxpayerData']['taxpayerAddress']['door']
				)
			);
			return $response;
		}
	}

	public function get_coupon_invoice_item_details($order) {
		$details = array(
			"title" => __('Kedvezmény', 'wc-szamlazz'),
			"desc" => ''
		);

		$order_discount = method_exists( $order, 'get_discount_total' ) ? $order->get_discount_total() : $order->order_discount;
		if ( $order_discount > 0 ) {
			$coupons = implode(', ', $order->get_used_coupons());
			$discount = strip_tags(html_entity_decode($order->get_discount_to_display()));
			$details["desc"] = sprintf( __( '%1$s kedvezmény a következő kupon kóddal: %2$s', 'wc-szamlazz' ), $discount, $coupons );

			if($this->get_option('wc_szamlazz_coupon_type_name')) {
				$details["title"] = $this->get_option('wc_szamlazz_coupon_type_name');
			}

			if($this->get_option('wc_szamlazz_coupon_type_desc')) {
				$discount_note_replacements = array('{kedvezmeny_merteke}' => $discount, '{kupon}' => $coupons);
				$discount_note = str_replace( array_keys( $discount_note_replacements ), array_values( $discount_note_replacements ), $this->get_option('wc_szamlazz_coupon_type_desc'));
				$details["desc"] = $discount_note;
			}
		}

		return $details;
	}

}

//WC Detection
if ( ! function_exists( 'is_woocommerce_active' ) ) {
	function is_woocommerce_active() {
		$active_plugins = (array) get_option( 'active_plugins', array() );

		if ( is_multisite() ) {
			$active_plugins = array_merge( $active_plugins, get_site_option( 'active_sitewide_plugins', array() ) );
		}

		return in_array( 'woocommerce/woocommerce.php', $active_plugins ) || array_key_exists( 'woocommerce/woocommerce.php', $active_plugins ) ;
	}
}


//WooCommerce inactive notice.
function wc_szamlazz_woocommerce_inactive_notice() {
	if ( current_user_can( 'activate_plugins' ) ) {
		echo '<div id="message" class="error"><p>';
		printf( __( 'A %1$sWooCommerce Számlázz.hu inaktív%2$s. A %3$sWooCommerce bővítménynek %4$s aktívnak kell lennie. %5$sTelepítsd és kapcsold be WooCommerce-et &raquo;%6$s', 'wc-szamlazz' ), '<strong>', '</strong>', '<a href="http://wordpress.org/extend/plugins/woocommerce/">', '</a>', '<a href="' . esc_url( admin_url( 'plugins.php' ) ) . '">', '</a>' );
		echo '</p></div>';
	}
}

//Initialize
if ( is_woocommerce_active() ) {
	$GLOBALS['wc_szamlazz'] = new WC_Szamlazz();
} else {
	add_action( 'admin_notices', 'wc_szamlazz_woocommerce_inactive_notice' );
}
